mod service_specific;
mod shared;

use std::{process::Command, time::Instant};

use anyhow::Result;
use service_specific::*;
use shared::config::{Config, COMMAND_COUNT, COMMAND_PAUSE};

fn main() -> Result<()> {
    println!("startup macroboard service");

    // azdelivery usart device
    // let usb_device = UsbId {
    //     vendor_id: 0x0403,
    //     product_id: 0x6001,
    // };

    // default arduino uno device
    let usb_device = UsbId {
        vendor_id: 0x2341,
        product_id: 0x0043,
    };

    // create serial port
    let mut port = Port::open(
        usb_device,
        SerialPortSettings {
            baud_rate: 57600,
            data_bits: DataBits::Eight,
            parity: Parity::None,
            stop_bits: StopBits::One,
            flow_control: FlowControl::None,
            timeout: Duration::from_millis(2500),
        },
    )?;

    let mut message_builder = MessageBuilder::default();
    let mut button_infos = [Duration::default(); COMMAND_COUNT];
    let start_time = Instant::now();

    println!("service is now listening ...");

    // loop forever
    loop {
        // handle incoming message
        match port.read()? {
            SerialReadResult::Message(msg) => {
                // create message out of stream
                // discard remaining for now
                if let Some(_remaining) = message_builder.check(msg) {
                    debug_assert!(message_builder.is_complete());

                    println!("{}", message_builder.message());

                    // if message can be parsed to a number, try to execute a command
                    if let Ok(index) = message_builder.message().parse() {
                        execute_button_press(index, start_time.elapsed(), &mut button_infos)?;
                    }

                    message_builder = MessageBuilder::default();
                }
            }
            SerialReadResult::UtfConversion(err) => println!("{:?}", err),
            SerialReadResult::Timeout => (),
        }
    }
}

fn execute_button_press(
    button_id: usize,
    time: Duration,
    button_infos: &mut [Duration; COMMAND_COUNT],
) -> Result<()> {
    // load config every button
    // since it could be modified from the gui application
    let config = Config::load_config()?;

    // check if ID is in a valid range
    if button_id < 1 && button_id > COMMAND_COUNT {
        return Ok(());
    }

    // check that this commands last execution is longer than COMMAND_PAUSE ago
    if (button_infos[button_id - 1] + COMMAND_PAUSE) < time {
        button_infos[button_id - 1] = time;
    } else {
        return Ok(());
    }

    // check command at ID position if a command is set
    // if so, simply execute this command as child process
    if let Some(cmd) = &config.commands[button_id - 1] {
        if !cmd.is_empty() {
            println!("cmd: {}", cmd);

            if let Err(err) = Command::new(cmd).spawn() {
                println!("failed with: {:?}", err);
            }
        }
    }

    Ok(())
}
